import sys
import os
sys.path.append("../")
import NEMtropy.graph_classes as sample
import NEMtropy.matrix_generator as mg
import numpy as np
import unittest  # test tool
import random
import networkx as nx


class MyTest(unittest.TestCase):


    def setUp(self):
        pass


    def test_0(self):
        N, seed = (5, 42)
        A = mg.random_binary_matrix_generator_dense(N, sym=False, seed=seed)
        # number of copies to generate

        g = sample.DirectedGraph(A)

        g._solve_problem(
            model="dcm",
            method="fixed-point",
            max_steps=100,
            verbose=False,
            linsearch=True,
            initial_guess="uniform",
        )

        # g._solution_error()
        err = g.error

        # print('\ntest 5: error = {}'.format(g.error))
        n = 100
        output_dir = "sample_dcm/"
        # random.seed(100)
        g.ensemble_sampler(n=n, output_dir=output_dir, seed=seed)

        d_out = {'{}'.format(i):g.dseq_out[i] for i in range(N)}
        d_in = {'{}'.format(i):g.dseq_in[i] for i in range(N)}


        # read all sampled graphs and check the average degree distribution is close enough
        d_out_emp = {'{}'.format(i):0 for i in range(N)}
        d_in_emp = {'{}'.format(i):0 for i in range(N)}

        for l in range(n):
            f = output_dir + "{}.txt".format(l)
            if not os.stat(f).st_size == 0:
                g_tmp = nx.read_edgelist(f, create_using=nx.DiGraph())
                d_out_tmp = dict(g_tmp.out_degree)
                d_in_tmp = dict(g_tmp.in_degree)
                for item in d_out_tmp.keys(): 
                    d_out_emp[item] += d_out_tmp[item]
                    d_in_emp[item] += d_in_tmp[item]


        for item in d_out_emp.keys(): 
            d_out_emp[item] = d_out_emp[item]/n
            d_in_emp[item] = d_in_emp[item]/n

        a_out_diff = np.array([abs(d_out[item] - d_out_emp[item]) for item in d_out.keys()])
        a_in_diff = np.array([abs(d_in[item] - d_in_emp[item]) for item in d_in.keys()])
        d_out_diff = {item:d_out[item] - d_out_emp[item] for item in d_out.keys()}
        d_in_diff = {item:d_in[item] - d_in_emp[item] for item in d_in.keys()}

        ensemble_error = np.linalg.norm(np.concatenate((a_out_diff, a_in_diff)), np.inf)

        #debug
        """
        for i in range(N):
            for j in range(N):
                if i!=j:
                    aux = x[i]*x[j]
                    # print("({},{}) p = {}".format(i,j,aux/(1+aux)))
        """


        # debug
        # print('original dseq',d_out,d_in)
        # print('original dseq out sum ',g.dseq_out.sum())
        # print('original dseq in sum ',g.dseq_in.sum())
        # print('ensemble average dseq out', d_out_emp)
        # print('ensemble average dseq in', d_in_emp)
        # print('ensemble dseq out sum ',np.array([d_out_emp[key] for key in d_out_emp.keys()]).sum())
        # print('ensemble dseq in sum ',np.array([d_in_emp[key] for key in d_in_emp.keys()]).sum())
        # print(d_out_diff)
        # print(d_in_diff)
        # print('empirical error', ensemble_error)
        # print('theoretical error', err)


        l = os.listdir(output_dir)
        for f in l:
            os.remove(output_dir + f)
        os.rmdir(output_dir)

        # test result
        self.assertTrue(ensemble_error < 3)


if __name__ == "__main__":
    unittest.main()



import sys
import os

sys.path.append("../")
import NEMtropy.graph_classes as sample
import NEMtropy.graph_classes as sample_und
import NEMtropy.matrix_generator as mg
import numpy as np
import unittest


class MyTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_crema_dcm_quasinewton_random_dense_20(self):

        network = mg.random_weighted_matrix_generator_dense(
            n=20, sup_ext=10, sym=False, seed=None
        )
        network_bin = (network > 0).astype(int)

        g = sample.DirectedGraph(adjacency=network)

        g.solve_tool(
            model="crema",
            method="newton",
            initial_guess = "random",
            adjacency="dcm",
            max_steps=1000,
            verbose=False,
        )

        g._solution_error()

        # test result

        self.assertTrue(g.relative_error_strength < 1e-1)
        self.assertTrue(g.relative_error_strength < 1e-2)

    def test_crema_original_quasinewton_random_dense_20_dir(self):

        network = mg.random_weighted_matrix_generator_dense(
            n=20, sup_ext=10, sym=False, seed=None
        )
        network_bin = (network > 0).astype(int)

        g = sample.DirectedGraph(adjacency=network)

        g.solve_tool(
            model="crema",
            method="newton",
            initial_guess = "random",
            adjacency=network_bin,
            max_steps=1000,
            verbose=False,
        )

        g._solution_error()

        # test result

        self.assertTrue(g.relative_error_strength < 1e-1)
        self.assertTrue(g.relative_error_strength < 1e-2)

    def test_crema_cm_quasinewton_random_dense_20(self):

        network = mg.random_weighted_matrix_generator_dense(
            n=20, sup_ext=10, sym=True, seed=None
        )
        network_bin = (network > 0).astype(int)

        g = sample_und.UndirectedGraph(adjacency=network)

        g.solve_tool(
            model="crema",
            method="newton",
            initial_guess = "random",
            adjacency="cm_exp",
            max_steps=1000,
            verbose=False,
        )

        g._solution_error()

        # test result

        self.assertTrue(g.relative_error_strength < 1e-1)
        self.assertTrue(g.relative_error_strength < 1e-2)

    def test_crema_original_quasinewton_random_dense_20_undir(self):

        network = mg.random_weighted_matrix_generator_dense(
            n=20, sup_ext=10, sym=False, seed=None
        )
        network_bin = (network > 0).astype(int)

        g = sample_und.UndirectedGraph(adjacency=network)

        g.solve_tool(
            model="crema",
            method="newton",
            initial_guess = "random",
            adjacency=network_bin,
            max_steps=1000,
            verbose=False,
        )

        g._solution_error()

        # test result

        self.assertTrue(g.relative_error_strength < 1e-1)
        self.assertTrue(g.relative_error_strength < 1e-2)


if __name__ == "__main__":

    unittest.main()



import sys

sys.path.append("../")
import NEMtropy.graph_classes as sample
import numpy as np
import unittest  # test tool
import NEMtropy.network_functions as mf
import os
import networkx as nx
import NEMtropy.matrix_generator as mg
import NEMtropy.ensemble_functions as en


class MyTest(unittest.TestCase):
    def setUp(self):
        pass

    @unittest.skip("works, need external data")
    def test_zscore_all_wtw(self):
        """This test use the wtw 2000 network, which is not usually loaded in the git repository and is used only for internal testing by the devs.
        """
        N, seed = (20, 100)
        # A = mg.random_binary_matrix_generator_dense(N, sym=False, seed=seed)
        A = np.loadtxt("A.txt")
        g = sample.DirectedGraph(A)

        g.solve_tool(
            model="dcm",
            max_steps=200,
            verbose=False,
        )

        d = g.zscore_3motifs()
        sol = np.concatenate((g.x, g.y))

        n = 100
        output_dir = "sample_dcm/"
        # random.seed(100)
        g.ensemble_sampler(n=n, output_dir=output_dir, seed=seed)
        n_sam = {str(i): np.zeros(n) for i in range(1,14)}
        for l in range(n):
            f = output_dir + "{}.txt".format(l)
            if not os.stat(f).st_size == 0:
                g_tmp = nx.read_edgelist(f, create_using=nx.DiGraph())
                a_tmp = nx.adjacency_matrix(g_tmp).toarray()
                for i in range(1, 14):
                    n_sam[str(i)][l] = eval(f'mf.count_3motif_{i}(a_tmp)')
				
        """
        z = {}
        for i in range(1, 14):
            n_emp = n_sam[str(i)]
            n = eval(f'mf.count_3motif_{i}(A)')
            n_emp_mu = np.mean(n_emp)
            n_emp_std = np.std(n_emp)
            n_mu = eval(f"en.expected_dcm_3motif_{i}(sol)")
            n_std = eval(f"en.std_dcm_3motif_{i}(sol)")
            # zz is d['13']
            zz = (n - n_mu)/n_std
            z[str(i)] = (n - n_emp_mu)/n_emp_std
            # debug
            print(f"motif {i}")
            print(f'n = {n}')
            print(f'm empirical mu = {n_emp_mu}')
            print(f'm analytical mu = {n_mu}')
            print(f'm empirical std = {n_emp_std}')
            print(f'm analytical std = {n_std}')
            print(f'empirical z-score = {z[str(i)]}')
            print(f"analytical z score = {d[str(i)]}")
            print(f"diff = d[{i}] - z = {abs(d[str(i)] - z[str(i)])}")
        """

        goal = {
            "1": -33.8,
            "2": -62.9,
            "3": -30.8,
            "4": -32.9,
            "5": -35.9,
            "6": -30.8,
            "7": -25.6,
            "8": 79.3,
            "9": -36,
            "10": -53.6,
            "11": -29.9,
            "12": -116.3,
            "13": 43.5
        }
        # debug
        # for i in range(1, 14):
        #     print(f"diff = d[{i}] - z = {abs(d[str(i)] - goal[str(i)])}")
        # test result
        for i in range(1, 14):
            print(i)
            self.assertTrue(abs(d[str(i)] - goal[str(i)])< 0.1)

        l = os.listdir(output_dir)
        for f in l:
            os.remove(output_dir + f)
        os.rmdir(output_dir)

    # @unittest.skip("debug")
    def test_zscore_all(self):
        N, seed = (10, 100)
        A = mg.random_binary_matrix_generator_dense(N, sym=False, seed=seed)
        g = sample.DirectedGraph(A)

        g.solve_tool(
            model="dcm",
            max_steps=200,
            verbose=False,
        )

        d = g.zscore_3motifs()
        sol = np.concatenate((g.x, g.y))

        n = 100
        output_dir = "sample_dcm/"
        # random.seed(100)
        g.ensemble_sampler(n=n, output_dir=output_dir, seed=seed)
        n_sam = {str(i): np.zeros(n) for i in range(1,14)}
        for l in range(n):
            f = output_dir + "{}.txt".format(l)
            if not os.stat(f).st_size == 0:
                g_tmp = nx.read_edgelist(f, create_using=nx.DiGraph())
                a_tmp = nx.adjacency_matrix(g_tmp).toarray()
                for i in range(1, 14):
                    n_sam[str(i)][l] = eval(f'mf.count_3motif_{i}(a_tmp)')
				
        z = {}
        for i in range(1, 14):
            n_emp = n_sam[str(i)]
            n = eval(f'mf.count_3motif_{i}(A)')
            n_emp_mu = np.mean(n_emp)
            n_emp_std = np.std(n_emp)
            n_mu = eval(f"en.expected_dcm_3motif_{i}(sol)")
            n_std = eval(f"en.std_dcm_3motif_{i}(sol)")
            # zz is d['13']
            zz = (n - n_mu)/n_std
            z[str(i)] = (n - n_emp_mu)/n_emp_std
            """
            # debug
            print(f"motif {i}")
            # print(f'n = {n}')
            # print(f'm empirical mu = {n_emp_mu}')
            # print(f'm analytical mu = {n_mu}')
            # print(f'm empirical std = {n_emp_std}')
            # print(f'm analytical std = {n_std}')
            # print(f'empirical z-score = {z[str(i)]}')
            # print(f"analytical z score = {d[str(i)]}")
            print(f"diff = d[{i}] - z = {abs(d[str(i)] - z[str(i)])}")
            """

        # test result
        #TODO: 5 is a bit too much of an interval
        for i in range(1, 14):
            self.assertTrue(abs(d[str(i)] - z[str(i)])< 5)

        l = os.listdir(output_dir)
        for f in l:
            os.remove(output_dir + f)
        os.rmdir(output_dir)


if __name__ == "__main__":
    unittest.main()



import sys

sys.path.append("../")
import NEMtropy.graph_classes as sample
import NEMtropy.graph_classes as sample_u
import NEMtropy.network_functions as ntw_f
import NEMtropy.matrix_generator as mg
import numpy as np
import unittest  # test tool


class MyTest(unittest.TestCase):
    def setUp(self):
        pass


    def test_cm_uniform(self):
        n, seed = (4, 22)
        A = mg.random_binary_matrix_generator_dense(n, sym=True, seed=seed)

        g = sample_u.UndirectedGraph(A)
        g.initial_guess = 'uniform'
        g.last_model = "cm"
        g._set_initial_guess('cm')

        self.assertTrue(g.x0.all() == np.array([0.5,  0.5]).all())


    def test_cm(self):
        n, seed = (4, 22)
        A = mg.random_binary_matrix_generator_dense(n, sym=True, seed=seed)

        x0 = np.random.rand(n)
        g = sample_u.UndirectedGraph(A)
        g.initial_guess = x0
        g._set_initial_guess_cm()
        g.full_return = False
        g.last_model = "cm"
        g._set_solved_problem_cm(g.x0)
        self.assertTrue(g.x.all() == x0.all())


    def test_crema_uniform(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        g = sample_u.UndirectedGraph(A)
        g.initial_guess = 'strengths_minor'
        g._set_initial_guess('crema')

        x = (g.strength_sequence > 0).astype(float) / (
                    g.strength_sequence + 1
                )
        self.assertTrue(g.x0.all() == x.all())


    def test_crema(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        x0 = np.random.rand(n)
        g = sample_u.UndirectedGraph(A)
        g.initial_guess = x0
        g._set_initial_guess_crema_undirected()
        g.full_return = False
        g._set_solved_problem_crema(g.x0)
        self.assertTrue(g.beta.all() == x0.all())


    def test_ecm_uniform(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        g = sample_u.UndirectedGraph(A)
        g.initial_guess = 'strengths_minor'
        g.last_model = "ecm"
        g._set_initial_guess('ecm')

        x = (g.strength_sequence > 0).astype(float) / (
                    g.strength_sequence + 1
                )
        self.assertTrue(g.x0.all() == x.all())


    def test_ecm(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        x0 = np.random.rand(n)
        g = sample_u.UndirectedGraph(A)
        g.initial_guess = x0
        g._set_initial_guess_crema_undirected()
        self.assertTrue(g.x0.all() == x0.all())



    def test_dcm_uniform(self):
        n, seed = (4, 22)
        A = mg.random_binary_matrix_generator_dense(n, sym=False, seed=seed)

        g = sample.DirectedGraph(A)
        g.initial_guess = 'uniform'
        g._set_initial_guess('dcm')
        self.assertTrue(np.concatenate((g.r_x, g.r_y)).all() == np.array([0.,  0.5, 0.5, 0.5, 0.,  0.5]).all())


    def test_dcm(self):
        n, seed = (4, 22)
        A = mg.random_binary_matrix_generator_dense(n, sym=False, seed=seed)

        x0 = np.random.rand(2*n)
        g = sample.DirectedGraph(A)
        g.initial_guess = x0
        g._set_initial_guess('dcm')
        g._set_solved_problem_dcm(x0)
        self.assertTrue(np.concatenate((g.x, g.y)).all() == x0.all())


    def test_dcm_exp_uniform(self):
        n, seed = (4, 22)
        A = mg.random_binary_matrix_generator_dense(n, sym=False, seed=seed)

        g = sample.DirectedGraph(A)
        g.initial_guess = 'uniform'
        g._set_initial_guess('dcm_exp')
        self.assertTrue(np.concatenate((g.r_x, g.r_y)).all() == np.array([1e3,  -np.log(0.5), -np.log(0.5), -np.log(0.5), 1e3,  -np.log(0.5)]).all())


    def test_dcm_exp(self):
        n, seed = (4, 22)
        A = mg.random_binary_matrix_generator_dense(n, sym=False, seed=seed)

        x0 = np.random.rand(2*n)
        g = sample.DirectedGraph(A)
        g.initial_guess = x0
        g._set_initial_guess('dcm_exp')
        g._set_solved_problem_dcm(x0)
        self.assertTrue(np.concatenate((g.x, g.y)).all() == x0.all())

    def test_decm_uniform(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )


        g = sample.DirectedGraph(A)
        g.initial_guess = 'uniform'
        g._set_initial_guess('decm')
        self.assertTrue(np.concatenate((g.x, g.y, g.out_strength, g.in_strength)).all() == np.ones(4*n).all())


    def test_decm(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        x0 = np.random.rand(4*n)
        g = sample.DirectedGraph(A)
        g.initial_guess = x0
        g._set_initial_guess('decm')
        g._set_solved_problem_decm(x0)
        self.assertTrue(np.concatenate((g.x, g.y)).all() == x0.all())



    def test_decm_exp_uniform(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )


        g = sample.DirectedGraph(A)
        g.initial_guess = 'uniform'
        g._set_initial_guess('decm_exp')
        tester = np.exp(np.ones(4*n))
        self.assertTrue(g.x0.all() == tester.all())


    def test_decm_exp(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        x0 = np.random.rand(4*n)
        g = sample.DirectedGraph(A)
        g.initial_guess = x0
        g._set_initial_guess('decm_exp')
        g._set_solved_problem_decm(x0)
        self.assertTrue(np.concatenate((g.x, g.y,g.out_strength, g.in_strength)).all() == x0.all())


    def test_crema_uniform(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        g = sample.DirectedGraph(A)
        g.initial_guess = 'strengths_minor'
        g._set_initial_guess_crema_directed()
        x = np.concatenate((ntw_f.out_strength(A)/(ntw_f.out_strength(A) + 1), ntw_f.in_strength(A)/(ntw_f.in_strength(A) + 1)))
        self.assertTrue(g.x0.all() == x.all())


    def test_crema(self):
        n, seed = (4, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        x0 = np.random.rand(2*n)
        g = sample.DirectedGraph(A)
        g.initial_guess = x0
        g._set_initial_guess_crema_directed()
        g._set_solved_problem_decm(x0)
        self.assertTrue(g.x0.all() == x0.all())


if __name__ == "__main__":
    unittest.main()



import sys

sys.path.append("../")
import NEMtropy.graph_classes as sample
import NEMtropy.models_functions as mof
import NEMtropy.matrix_generator as mg
import numpy as np
import unittest  # test tool
from scipy.optimize import approx_fprime


class MyTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_loglikelihood_dcm(self):
        """
        a = np.array([[0, 1, 1],
                      [1, 0, 1],
                      [0, 1, 0]])
        """
        n, seed = (3, 42)
        a = mg.random_binary_matrix_generator_dense(n, sym=False, seed=seed)

        g = sample.DirectedGraph(a)
        g.degree_reduction()
        g.initial_guess = "uniform"
        g.regularise = "eigenvalues"
        g._initialize_problem("dcm", "quasinewton")
        x0 = np.concatenate((g.r_x, g.r_y))

        # call loglikelihood function
        f_sample = -g.step_fun(x0)
        f_correct = 4 * np.log(1 / 2) - 3 * np.log(5 / 4)
        # debug
        # print(x0)
        # print(f_sample)
        # print(f_correct)

        # test result
        self.assertTrue(round(f_sample, 3) == round(f_correct, 3))

    def test_loglikelihood_prime_dcm(self):
        """
        a = np.array([[0, 1, 1],
                      [1, 0, 1],
                      [0, 1, 0]])
        """
        n, seed = (30, 42)
        a = mg.random_binary_matrix_generator_dense(n, sym=False, seed=seed)

        # rd
        g = sample.DirectedGraph(a)
        g.degree_reduction()
        g.initial_guess = "uniform"
        g.regularise = "eigenvalues"
        g._initialize_problem("dcm", "newton")

        k_out = g.args[0]
        k_in = g.args[1]
        nz_index_out = g.args[2]
        nz_index_in = g.args[3]

        n_rd = len(k_out)
        theta = np.random.rand(2 * n_rd)
        f_sample = np.zeros(n_rd * 2)
        f = lambda x: mof.loglikelihood_dcm(x, g.args)
        f_sample = approx_fprime(theta, f, epsilon=1e-6)
        f_new = mof.loglikelihood_prime_dcm(theta, g.args)

        # debug
        # print(a)
        # print(x0, x)
        # print(f_sample)
        # print(f_new)
        # for i in range(2*n_rd):
        #         if not np.allclose(f_new[i], f_sample[i],atol=1e-1):
        #             print(i)

        # test result
        self.assertTrue(np.allclose(f_sample, f_new, atol=1e-1))

    def test_loglikelihood_hessian_dcm(self):

        # n,s =(3, 1)
        n, s = (30, 1)
        a = mg.random_binary_matrix_generator_custom_density(
            n=n, p=0.15, sym=False, seed=s
        )

        g = sample.DirectedGraph(a)
        g.degree_reduction()
        g.initial_guess = "uniform"
        g.regularise = "eigenvalues"
        g._initialize_problem("dcm", "newton")

        k_out = g.args[0]
        k_in = g.args[1]
        nz_index_out = g.args[2]
        nz_index_in = g.args[3]

        n_rd = len(k_out)
        theta = np.random.rand(2 * n_rd)
        f_sample = np.zeros((n_rd * 2, n_rd * 2))
        for i in range(n_rd * 2):
            f = lambda x: mof.loglikelihood_prime_dcm(x, g.args)[i]
            f_sample[i, :] = approx_fprime(theta, f, epsilon=1e-6)

        f_new = mof.loglikelihood_hessian_dcm(theta, g.args)

        """
        for i in range(2*n_rd):
            for j in range(2*n_rd):
                if np.allclose(f_new[i,j], f_sample[i,j], atol=1e-1) == False:
                    print(i,j)
                    print(f_new[i,j])
                    print(f_sample[i,j])
                    print(f_sample[i,j]/f_new[i,j])
        """

        # debug
        # print(theta, x0)
        # print(g.args)
        # print(n_rd/n)
        # print(f_sample)
        # print(f_new)

        # test result
        self.assertTrue(np.allclose(f_sample, f_new, atol=1))

    def test_loglikelihood_hessian_diag_dcm(self):
        a = np.array([[0, 1, 1], [1, 0, 1], [0, 1, 0]])
        k_out = np.sum(a > 0, 1)
        k_in = np.sum(a > 0, 0)
        nz_ind_out = np.nonzero(k_out)[0]
        nz_ind_in = np.nonzero(k_in)[0]
        c = np.array([1, 1, 1])
        args = (k_out, k_in, nz_ind_out, nz_ind_in, c)
        x = 0.5 * np.ones(len(k_out) + len(k_in))
        # call loglikelihood function
        f_sample = mof.loglikelihood_hessian_diag_dcm(x, args)
        # debug
        # print(par)
        # print(f_sample)
        # print(f_correct)

        # test result
        # self.assertTrue(np.allclose(f_sample, f_correct))

    def test_iterative_dcm(self):

        n, seed = (3, 42)
        a = mg.random_binary_matrix_generator_dense(n, sym=False, seed=seed)

        # rd
        g = sample.DirectedGraph(a)
        g.degree_reduction()
        g.initial_guess = "uniform"
        g.regularise = "eigenvalues"
        g._initialize_problem("dcm", "fixed-point")
        x0 = 0.5 * np.ones(4)

        f_sample = -g.fun(x0)
        g.last_model = "dcm"
        g._set_solved_problem(f_sample)
        f_full = np.concatenate((g.x, g.y))
        f_correct = -np.array([2.5, 2.5, 0, 0, 1, 1.25])

        # debug
        # print(a)
        # print(x0, x)
        # print(f_full)

        # test result
        self.assertTrue(np.allclose(f_full, f_correct))

    def test_iterative_dcm_1(self):
        degseq = np.array([0, 1, 2, 1, 2, 2, 2, 0, 2, 0])

        # rd
        g = sample.DirectedGraph(degree_sequence=degseq)
        g.degree_reduction()
        g.initial_guess = "uniform"
        g.regularise = "eigenvalues"
        g._initialize_problem("dcm", "fixed-point")
        x0 = np.ones(6)
        # x0[x0 == 0] = 0

        f_sample = -g.fun(x0)
        # g._set_solved_problem(f_sample)
        # f_full = np.concatenate((g.x, g.y))
        f_correct = -np.array([0, 0.5, 1, 1, 1, 0])

        # debug
        # print(g.args)
        # print(f_sample)
        # print(f_correct)

        # test result
        self.assertTrue(np.allclose(f_sample, f_correct))

    def test_loglikelihood_hessian_dcm_vs_diag(self):
        a = np.array([[0, 1, 1], [1, 0, 1], [0, 1, 0]])
        k_out = np.sum(a > 0, 1)
        k_in = np.sum(a > 0, 0)
        nz_ind_out = np.nonzero(k_out)[0]
        nz_ind_in = np.nonzero(k_in)[0]
        c = np.array([1, 1, 1])
        args = (k_out, k_in, nz_ind_out, nz_ind_in, c)
        x = 0.5 * np.ones(len(k_out) + len(k_in))
        # call loglikelihood function
        f_diag = mof.loglikelihood_hessian_diag_dcm(x, args)
        f_full = mof.loglikelihood_hessian_dcm(x, args)
        f_df = np.diag(f_full)
        # debug
        # print(f_diag, f_full, f_df)

        # test result
        self.assertTrue(np.allclose(f_diag, f_df))


if __name__ == "__main__":
    unittest.main()



import sys

sys.path.append("../")
import NEMtropy.graph_classes as sample
import NEMtropy.matrix_generator as mg
import numpy as np
import unittest  # test tool


class MyTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_0(self):
        n, seed = (14, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        g = sample.DirectedGraph(A)

        g._solve_problem(
            model="decm_exp",
            method="quasinewton",
            max_steps=3000,
            verbose=False,
            initial_guess="uniform",
            linsearch=True,
        )

        g._solution_error()
        # debug
        # print("\n test 0, no zeros, dimension n = {}, error = {}".format(n, g.error))

        # test result
        self.assertTrue(g.error < 1)

    def test_1(self):
        # test Matrix 1
        n, seed = (14, 22)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )
        A[0, :] = 0

        g = sample.DirectedGraph(A)

        g._solve_problem(
            model="decm_exp",
            method="quasinewton",
            max_steps=3000,
            verbose=False,
            initial_guess="uniform",
            linsearch=True,
        )

        g._solution_error()
        # debug
        # print("\n test 1, zeros, dimension n = {}, error = {}".format(n, g.error))

        # test result
        self.assertTrue(g.error < 1)

    @unittest.skip("skip large graph")
    def test_2(self):
        # test Matrix 1
        n, seed = (40, 35)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )

        g = sample.DirectedGraph(A)

        g._solve_problem(
            model="decm_exp",
            method="quasinewton",
            max_steps=25000,
            verbose=False,
            initial_guess="uniform",
            linsearch=True,
        )

        g._solution_error()
        # debug
        print(
            "\n test 2, no zeros, dimension n = {}, error = {}".format(
                n, g.error
            )
        )

        # test result
        self.assertTrue(g.error < 1)

    @unittest.skip("skip large graph")
    def test_3(self):
        n, seed = (40, 35)
        A = mg.random_weighted_matrix_generator_dense(
            n, sym=False, seed=seed, sup_ext=100, intweights=True
        )
        A[0, :] = 0

        g = sample.DirectedGraph(A)

        g._solve_problem(
            model="decm_exp",
            method="quasinewton",
            max_steps=20000,
            verbose=False,
            initial_guess="uniform",
            linsearch=True,
        )

        g._solution_error()
        # debug
        print(
            "\n test 3, zeros, dimension n = {}, error = {}".format(n, g.error)
        )

        # test result
        self.assertTrue(g.error < 1)


if __name__ == "__main__":
    unittest.main()



import sys

sys.path.append("../")
import NEMtropy.graph_classes as sample
import numpy as np
import unittest  # test tool
import NEMtropy.matrix_generator as mg
import NEMtropy.models_functions as mof


class MyTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_dcm(self):
        """test with 3 classes of cardinality 1
        and no zero degrees
        """
        A = np.array(
            [
                [0, 1, 1],
                [1, 0, 0],
                [0, 1, 0],
            ]
        )

        g = sample.DirectedGraph(A)

        g.solve_tool(
            model="dcm",
            max_steps=200,
            verbose=False,
        )
        l = -mof.loglikelihood_dcm(g.solution_array, g.args)

        # g._solution_error()
        # debug
        # print(g.r_dseq_out)
        # print(g.r_dseq_in)
        # print(g.rnz_dseq_out)
        # print(g.rnz_dseq_in)
        # print('\ntest 0: error = {}'.format(g.error))

        # test result
        self.assertTrue(l == g.model_loglikelihood())

    def test_dcm_new(self):
        """test with 3 classes of cardinality 1
        and no zero degrees
        """
        A = np.array(
            [
                [0, 1, 1],
                [1, 0, 0],
                [0, 1, 0],
            ]
        )

        g = sample.DirectedGraph(A)

        g.solve_tool(
            model="dcm_exp",
            method="quasinewton",
            initial_guess="uniform",
            max_steps=200,
            verbose=False,
        )
        l = -mof.loglikelihood_dcm_exp(g.solution_array, g.args)

        # g._solution_error()
        # debug
        # print(g.r_dseq_out)
        # print(g.r_dseq_in)
        # print(g.rnz_dseq_out)
        # print(g.rnz_dseq_in)
        # print('\ntest 0: error = {}'.format(g.error))
        # print(l)
        # print(g.model_loglikelihood())

        # test result
        self.assertTrue(l == g.model_loglikelihood())

    @unittest.skip("")
    def test_decm(self):
        """test with 3 classes of cardinality 1
        and no zero degrees
        """

        n, s = (4, 25)

        A = mg.random_weighted_matrix_generator_dense(
            n, sup_ext=10, sym=False, seed=s, intweights=True
        )

        g = sample.DirectedGraph(A)

        g.solve_tool(
            model="decm",
            method="quasinewton",
            initial_guess="uniform",
            max_steps=200,
            verbose=False,
        )

        l = -mof.loglikelihood_decm(g.solution_array, g.args)

        # g._solution_error()
        # debug
        # print(g.r_dseq_out)
        # print(g.r_dseq_in)
        # print(g.rnz_dseq_out)
        # print(g.rnz_dseq_in)
        # print('\ntest 0: error = {}'.format(g.error))

        # test result
        self.assertTrue(l == g.model_loglikelihood())

    def test_decm_new(self):
        """test with 3 classes of cardinality 1
        and no zero degrees
        """

        n, s = (10, 25)

        A = mg.random_weighted_matrix_generator_dense(
            n, sup_ext=10, sym=False, seed=s, intweights=True
        )

        g = sample.DirectedGraph(A)

        g.solve_tool(
            model="decm_exp",
            method="quasinewton",
            initial_guess="uniform",
            max_steps=200,
            verbose=False,
        )
        l = -mof.loglikelihood_decm_exp(g.solution_array, g.args)

        # g._solution_error()
        # debug
        # print(g.r_dseq_out)
        # print(g.r_dseq_in)
        # print(g.rnz_dseq_out)
        # print(g.rnz_dseq_in)
        # print('\ntest 0: error = {}'.format(g.error))

        # test result
        self.assertTrue(l == g.model_loglikelihood())


if __name__ == "__main__":
    unittest.main()



import sys

sys.path.append("../")
import NEMtropy.graph_classes as sample
from NEMtropy.models_functions import *
import NEMtropy.matrix_generator as mg
import numpy as np
import unittest  # test tool
from scipy.optimize import approx_fprime


class MyTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_iterative_decm_exp(self):

        A = np.array([[0, 2, 2], [2, 0, 2], [0, 2, 0]])

        bA = np.array([[1 if aa != 0 else 0 for aa in a] for a in A])

        k_out = np.sum(bA, axis=1)
        k_in = np.sum(bA, axis=0)
        s_out = np.sum(A, axis=1)
        s_in = np.sum(A, axis=0)

        # rd
        g = sample.DirectedGraph(A)
        g.initial_guess = "uniform"
        g.regularise = "identity"
        g._initialize_problem("decm", "fixed-point")
        # theta = np.random.rand(6)
        theta = 0.5 * np.ones(12)
        x0 = np.exp(-theta)

        f_sample = g.fun(x0)
        f_full = -np.log(f_sample)
        f_exp = iterative_decm_exp(theta, g.args)

        # f_exp_bis = iterative_dcm_exp_bis(theta, g.args)
        # print('normale ',f_exp)
        # print('bis',f_exp_bis)

        # debug
        # print(a)
        # print(theta, x0)
        # print(g.args)
        # print(f_exp)
        # print(f_full)

        # test result
        # first halves are same, second not
        self.assertTrue(np.allclose(f_full, f_exp))

    def test_loglikelihood_dcm_exp(self):

        A = np.array([[0, 2, 2], [2, 0, 2], [0, 2, 0]])

        bA = np.array([[1 if aa != 0 else 0 for aa in a] for a in A])

        k_out = np.sum(bA, axis=1)
        k_in = np.sum(bA, axis=0)
        s_out = np.sum(A, axis=1)
        s_in = np.sum(A, axis=0)

        g = sample.DirectedGraph(A)
        g.initial_guess = "uniform"
        g.regularise = "identity"
        g._initialize_problem("decm", "newton")
        # theta = np.random.rand(6)
        theta = 0.5 * np.ones(12)
        x0 = np.exp(-theta)

        f_sample = g.step_fun(x0)
        g.last_model = "decm"
        f_exp = -loglikelihood_decm_exp(theta, g.args)

        # debug
        # print(a)
        # print(theta, x0)
        # print(g.args)
        # print(f_sample)
        # print(f_sample)
        # print(f_exp)

        # test result
        self.assertTrue(
            np.round(f_sample, decimals=5) == np.round(f_exp, decimals=5)
        )

    def test_loglikelihood_prime_dcm_exp(self):

        A = np.array([[0, 2, 2], [2, 0, 2], [0, 2, 0]])

        bA = np.array([[1 if aa != 0 else 0 for aa in a] for a in A])

        k_out = np.sum(bA, axis=1)
        k_in = np.sum(bA, axis=0)
        s_out = np.sum(A, axis=1)
        s_in = np.sum(A, axis=0)

        g = sample.DirectedGraph(A)
        g.initial_guess = "uniform"
        g.regularise = "identity"
        g._initialize_problem("decm", "newton")
        # theta = np.random.rand(6)
        theta = 0.5 * np.ones(12)
        x0 = np.exp(-theta)

        f = lambda x: loglikelihood_decm_exp(x, g.args)
        f_sample = approx_fprime(theta, f, epsilon=1e-6)
        f_exp = loglikelihood_prime_decm_exp(theta, g.args)

        # debug
        # print(a)
        # print(theta, x0)
        # print(g.args)
        # print(f_sample)
        # print(f_exp)

        # test result
        self.assertTrue(np.allclose(f_sample, f_exp))

    def test_loglikelihood_hessian_decm_exp(self):

        A = np.array([[0, 2, 2], [2, 0, 2], [0, 2, 0]])

        bA = np.array([[1 if aa != 0 else 0 for aa in a] for a in A])

        k_out = np.sum(bA, axis=1)
        k_in = np.sum(bA, axis=0)
        s_out = np.sum(A, axis=1)
        s_in = np.sum(A, axis=0)

        g = sample.DirectedGraph(A)
        g.initial_guess = "uniform"
        g.regularise = "identity"
        g._initialize_problem("decm", "newton")
        # theta = np.random.rand(6)
        theta = 0.5 * np.ones(12)
        x0 = np.exp(-theta)

        f_sample = np.zeros((12, 12))
        for i in range(12):
            f = lambda x: loglikelihood_prime_decm_exp(x, g.args)[i]
            f_sample[i, :] = approx_fprime(theta, f, epsilon=1e-6)

        f_exp = loglikelihood_hessian_decm_exp(theta, g.args)

        # debug
        # print(a)
        # print(theta, x0)
        # print(g.args)
        # print('approx',f_sample)
        # print('my',f_exp)
        # print('diff',f_sample - f_exp)
        # print('max',np.max(np.abs(f_sample - f_exp)))

        # test result
        self.assertTrue(np.allclose(f_sample, f_exp))

    def test_loglikelihood_hessian_diag_decm_exp(self):

        A = np.array([[0, 2, 2], [2, 0, 2], [0, 2, 0]])

        bA = np.array([[1 if aa != 0 else 0 for aa in a] for a in A])

        k_out = np.sum(bA, axis=1)
        k_in = np.sum(bA, axis=0)
        s_out = np.sum(A, axis=1)
        s_in = np.sum(A, axis=0)

        g = sample.DirectedGraph(A)
        g.initial_guess = "uniform"
        g.regularise = "identity"
        g._initialize_problem("decm", "newton")
        # theta = np.random.rand(6)
        theta = 0.5 * np.ones(12)
        x0 = np.exp(-theta)

        f_sample = np.zeros(12)
        for i in range(12):
            f = lambda x: loglikelihood_prime_decm_exp(x, g.args)[i]
            f_sample[i] = approx_fprime(theta, f, epsilon=1e-6)[i]

        f_exp = loglikelihood_hessian_diag_decm_exp(theta, g.args)

        # debug
        # print(a)
        # print(theta, x0)
        # print(g.args)
        # print('approx',f_sample)
        # print('my',f_exp)
        # print('diff',f_sample - f_exp)
        # print('max',np.max(np.abs(f_sample - f_exp)))

        # test result
        self.assertTrue(np.allclose(f_sample, f_exp))

    def test_loglikelihood_hessian_diag_dcm_exp_zeros(self):

        # convergence relies heavily on x0
        n, s = (10, 35)
        # n, s = (5, 35)
        A = mg.random_weighted_matrix_generator_dense(
            n, sup_ext=100, sym=False, seed=s, intweights=True
        )
        A[0, :] = 0
        A[:, 5] = 0

        bA = np.array([[1 if aa != 0 else 0 for aa in a] for a in A])

        k_out = np.sum(bA, axis=1)
        k_in = np.sum(bA, axis=0)
        s_out = np.sum(A, axis=1)
        s_in = np.sum(A, axis=0)

        g = sample.DirectedGraph(A)
        g.initial_guess = "uniform"
        g.regularise = "identity"
        g._initialize_problem("decm", "newton")
        # theta = np.random.rand(6)
        theta = 0.5 * np.ones(n * 4)
        theta[np.concatenate((k_out, k_in, s_out, s_in)) == 0] = 1e4

        x0 = np.exp(-theta)

        f_sample = np.zeros(n * 4)
        for i in range(n * 4):
            f = lambda x: loglikelihood_prime_decm_exp(x, g.args)[i]
            f_sample[i] = approx_fprime(theta, f, epsilon=1e-6)[i]

        f_exp = loglikelihood_hessian_diag_decm_exp(theta, g.args)

        # debug
        # print(a)
        # print(theta, x0)
        # print(g.args)
        # print('approx',f_sample)
        # print('my',f_exp)
        # print('gradient', loglikelihood_prime_decm_exp(theta, g.args))
        # print('diff',f_sample - f_exp)
        # print('max',np.max(np.abs(f_sample - f_exp)))

        # test result
        self.assertTrue(np.allclose(f_sample, f_exp))


if __name__ == "__main__":
    unittest.main()



import sys

sys.path.append("../")
import NEMtropy.graph_classes as sample
import numpy as np
import unittest  # test tool


class MyTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_adjacency_init(self):
        a = np.array([[0, 1, 1], [1, 0, 1], [0, 1, 0]])
        k_out = np.array([2, 2, 1])
        k_in = np.array([1, 2, 2])

        g0 = sample.DirectedGraph()
        g0._initialize_graph(a)
        # debug
        # test result
        self.assertTrue((k_out == g0.dseq_out).all())
        self.assertTrue((k_in == g0.dseq_in).all())

    def test_edgelist_init(self):
        e = np.array([(0, 1), (0, 2), (1, 2), (1, 0), (2, 1)])
        k_out = np.array([2, 2, 1])
        k_in = np.array([1, 2, 2])

        g0 = sample.DirectedGraph()
        g0._initialize_graph(edgelist=e)
        # debug
        # test result

        self.assertTrue((k_out == g0.dseq_out).all())
        self.assertTrue((k_in == g0.dseq_in).all())

    def test_edgelist_init_string_undirected(self):
        e = np.array([("1", "a"), ("2", "b"), ("2", "a")])
        k = np.array([1, 2, 2, 1])

        g0 = sample.UndirectedGraph()
        g0._initialize_graph(edgelist=e)
        # debug
        # test result
        self.assertTrue(k.all() == g0.dseq.all())

    def test_edgelist_init_string_undirected_weighted(self):
        e = np.array([("1", "a", 3), ("2", "b", 4), ("2", "a", 3)])
        k = np.array([1, 2, 2, 1])
        s = np.array([3., 7., 6., 4.])

        g0 = sample.UndirectedGraph()
        g0._initialize_graph(edgelist=e)
        # debug
        # test result
        self.assertTrue((k == g0.dseq).all())
        self.assertTrue((s == g0.strength_sequence).all())

    def test_edgelist_init_string_directed(self):
        e = np.array([("1", "a"), ("2", "b"), ("2", "a")])
        k_out = np.array([1, 2, 0, 0])
        k_in = np.array([0, 0, 2, 1])

        g0 = sample.DirectedGraph()
        g0._initialize_graph(edgelist=e)
        # debug
        # test result
        self.assertTrue((k_out == g0.dseq_out).all())
        self.assertTrue((k_in == g0.dseq_in).all())

    def test_edgelist_init_string_directed_weighted(self):
        e = np.array([("1", "a", 3), ("2", "b", 4), ("2", "a", 3)])
        k_out = np.array([1, 2, 0, 0])
        k_in = np.array([0, 0, 2, 1])

        s_out = np.array([3., 7., 0., 0.])
        s_in = np.array([0., 0., 6., 4.])

        g0 = sample.DirectedGraph()
        g0._initialize_graph(edgelist=e)
        # debug
        # test result
        self.assertTrue((k_out == g0.dseq_out).all())
        self.assertTrue((k_in == g0.dseq_in).all())
        self.assertTrue((s_out == g0.out_strength).all())
        self.assertTrue((s_in == g0.in_strength).all())


if __name__ == "__main__":
    unittest.main()



import sys

sys.path.append("../")
import NEMtropy.graph_classes as sample
import numpy as np
import unittest  # test tool


class MyTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_dcm(self):
        A = np.array([[0, 1, 1, 0], [1, 0, 1, 0], [1, 0, 0, 1], [0, 1, 0, 0]])

        g = sample.DirectedGraph()
        g._initialize_graph(A)

        g.degree_reduction()

        g.initial_guess = "degrees"
        g._set_initial_guess("dcm")

        # debug
        # print(g.r_dseq_out)
        # print(g.r_dseq_in)
        # print(g.rnz_dseq_out)
        # print(g.rnz_dseq_in)

        sol = np.concatenate((g.r_x, g.r_y))
        g.last_model = "dcm"
        g._set_solved_problem(sol)

        # test result
        self.assertTrue(g.dseq_out.all() == g.x.all())
        self.assertTrue(g.dseq_in.all() == g.y.all())


if __name__ == "__main__":
    unittest.main()
